<!-- NOTE: It is safe to edit this file. -->
<!-- This file is only initially generated by aac gen-plugin, and it won't be overwritten if the file already exists. -->
<!-- Be sure to remove these comments as this file will be used for publishing to site documentation -->

# {{ plugin.name }}

{{ plugin.description }}

for command in plugin.commands
## {{ title(command.name) }}
```bash
aac {{ command.name }} example_file.aac
```

{{ command.description }}

## Arguments

for argument in command.input
### {{ title(argument) }} <!-- title would be a helper function to get proper formatted name -->

{{ argument.description }}
// end arguments for loop

## Help
```bash
<!-- Include a terminal output of executing the help flag on the plugin command -->
```

## Examples
For viewing example input and output of executing this command, view [{{ title(argument)}}](command example page)
<!-- Be sure to include basic execution, complex execution using any optional flags, and failures of command execution. -->
// end command for loop
