"""Constants for IO-related items."""
from os import linesep

YAML_DOCUMENT_SEPARATOR = "---"
DEFINITION_SEPARATOR = f"{YAML_DOCUMENT_SEPARATOR}{linesep}"
DEFAULT_SOURCE_URI = "<string>"
YAML_DOCUMENT_EXTENSION = ".yaml"
AAC_DOCUMENT_EXTENSION = ".aac"
