[build-system]
requires = ["setuptools>=61.0"]
build-backend = "setuptools.build_meta"

[project]
name = "{{project.name}}"
version = "0.0.1"
authors = [
	{ name="AaC Dev Team", email="asdfasdaf@email.com" },
]
description = "{{ project.description }}"
requires-python = ">= 3.9.13"
readme = "README.md"
license = {text = "MIT License"}
keywords = ["SysML", "Architecture-as-Code", "System Engineering", "MBSE"]
classifiers = [
	"Programming Language :: Python :: 3",
	"License :: OSI Approved :: MIT License",
	"Operating System :: OS Independent",
	"Topic :: Scientific/Engineering"
]

dependencies = [
    "aac ~= {{ aac_version() }}",
    {% for uses in project.uses %}"{{uses.name}} ~= {{uses.version}}",
    {% endfor %}
]
[project.optional-dependencies]
doc = [
	"sphinx >= 7.2.6",
	"sphinx-autobuild ~= 2021.3.14",
	"sphinx_contributors >= 0.2.7",
	"sphinx-copybutton >= 0.5.2",
	"sphinxcontrib-applehelp ~= 1.0.2",
    "sphinxcontrib-devhelp ~= 1.0.2",
    "sphinxcontrib-htmlhelp ~= 2.0.0",
    "sphinxcontrib-jsmath ~= 1.0.1",
    "sphinxcontrib-qthelp ~= 1.0.3",
    "sphinxcontrib-serializinghtml ~= 1.1.9",
	"docutils ~= 0.19",
	"furo >= 2024.1.29",
	"myst-parser >= 2.0.0",
	"pytz ~= 2023.3"
]
test = [
	"behave >= 1.2.6",
	"build >= 1.1.1",
	"coverage >= 7.4.3",
	"flake8 >= 4.0.0",
	"nose2 >= 0.10.0",
	"pipdeptree >= 2.16.1",
	"pyflakes >= 2.4",
	"tox >= 3.24",
    "httpx ~= 0.27.0"
]

all = [
	"aac >= 0.4.8",
	"{{project.name}}[doc]",
	"{{project.name}}[test]"
]

[project.entry-points."aac"]
{{project.name}} = "{{get_python_name(project.name)}}"
[project.urls]
Homepage = "https://github.com/DevOps-MBSE/{{project.name}}"
Issues = "https://github.com/DevOps-MBSE/{{project.name}}/issues"

[tool.setuptools]
include-package-data = true
package-dir = {"" = "src"}
[tool.setuptools.packages.find]
where = ["src"]
exclude = ["tests"]
[tool.setuptools.package-data]
mypkg = ["**/*.aac", "**/*.jinja2", "*.yaml"]
