<!-- NOTE: It is safe to edit this file. -->
<!-- This file is only initially generated by aac gen-plugin, and it won't be overwritten if the file already exists. -->
<!-- Be sure to remove these comments as this file will be used for publishing to site documentation -->

# {{ title(command.name) }}

## Basic Command execution
```bash
<!-- Include terminal output of executing the command without optional flags -->
```

### Basic Command Output
<!-- Include any file outputs -->

for argument in command.input if argument.startswith("--")
## Command Execution with {{ argument }}
```bash
<!-- Include terminal output of executing the command with first optional flag -->
```

### Command Execution with {{ argument }} Output
<!-- Include any file outputs -->
// end optional argument loop

## Command Execution Failure
```bash
<!-- Include terminal output of execution of the command failing -->
```

if output files are generated
## Generated Files
<!-- Include a screen shot of updated file tree -->
// end if loop
